#ifndef PARTICLE_PP_H_
#define PARTICLE_PP_H_

#include <ppapi/cpp/graphics_2d.h>
#include <ppapi/cpp/image_data.h>
#include <ppapi/cpp/instance.h>
#include <ppapi/cpp/rect.h>
#include <ppapi/cpp/dev/scriptable_object_deprecated.h>
#include <ppapi/cpp/size.h>
#include <pthread.h>
#include <string.h>

#include <ppapi/cpp/completion_callback.h>
#include <ppapi/cpp/module.h>
#include <ppapi/cpp/var.h>
#include <cstdio>
#include <string>
#include <map>
#include <vector>

namespace particle_pp {
#define RANDOM() (static_cast<double>(rand()) / RAND_MAX)

union rgb32 {
	uint32_t	rgba;
	struct _b {
		unsigned char b,g,r,a;
	} b;
};

	class ParticlePpInstance : public pp::Instance {
	protected:
		pp::Graphics2D				*graphics_2d_context_;
		pp::ImageData				*pixel_buffer_;
		int							particle_count;
		std::vector<float>			*particles;
	private:
		void						CreateContext(const pp::Size& size);
		void						DestroyContext();
		pp::Var						FlushPixelBuffer();
		bool IsContextValid() const {
			return graphics_2d_context_ != NULL;
		}

		bool						flush_pending_;
	public:
		explicit ParticlePpInstance(PP_Instance instance);
		virtual ~ParticlePpInstance();
		
		virtual bool				Init(uint32_t argc, const char* argn[], const char* argv[]);
		virtual void				DidChangeView(const pp::Rect& position, const pp::Rect& clip);
		virtual pp::Var				GetInstanceObject();
		//uint32_t					*LockPixels();
		//void						UnlockPixels() const;
		pp::Var						Paint();

		int width() const {
			return pixel_buffer_ ? pixel_buffer_->size().width() : 0;
		}
		int height() const {
			return pixel_buffer_ ? pixel_buffer_->size().height() : 0;
		}
		bool flush_pending() const {
			return flush_pending_;
		}
		void set_flush_pending(bool flag) {
			flush_pending_ = flag;
		}

		pp::Var						ParticleInit(int count);
		virtual pp::Var				updateParticlesSSE2(float mx, float my, float force, bool colorTransform);
		virtual pp::Var				updateParticles(float mx, float my, float force, bool colorTransform);

	};

}
#endif
