// Copyright 2010 The Native Client Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can
// be found in the LICENSE file.

#include "particle_pp.h"
#include "particle_pp_SSE2.h"

namespace particle_pp {

	ParticlePpInstanceSSE2::ParticlePpInstanceSSE2(PP_Instance instance)
	: ParticlePpInstance(instance)
	{
	}

	typedef float v4sf __attribute__ ((vector_size(16))) __attribute__((aligned(16)));
	typedef int   v4si __attribute__ ((vector_size(16))) __attribute__((aligned(16)));
	typedef double v2df __attribute__ ((vector_size(16))) __attribute__((aligned(16)));

	typedef union _vec2 {
		v2df v;
		double a[2];
	} vec2;

	typedef union _vec4 {
		v4sf v;
		float a[4];
	} vec4;

	typedef union _vec4i {
		v4si v;
		int a[4];
	} vec4i;

		
	#define GROUP4
	
	pp::Var ParticlePpInstanceSSE2::updateParticlesSSE2(float mx, float my, float force, bool colorTransform)
	{
		if (!particles) return pp::Var(0.0);

		uint32_t* pixel_bits =  reinterpret_cast<uint32_t*>(pixel_buffer_->data());
		if (pixel_bits == NULL) return pp::Var(0.0);

		float gravPoint_x = mx + RANDOM()*16;
		float gravPoint_y = my + RANDOM()*16;
		
		int w = width();
		int h = height();
		
		if (colorTransform) {
			v4sf red  = { 0.7,  0.8, 0.96, 1.0 };
			v4sf blue = { 0.96, 0.8, 0.70, 1.0 };
			v4sf coeff;
			if (force > 0) {
				coeff = blue;
			} else {
				coeff = red;
			}

			uint32_t* p = pixel_bits;
			for (int y = 0; y < h; y++) {
				for (int x = 0; x < w; x++) {
					// ܂SSE2̌ʂȂB̃[hHvȂႢȂH
					union rgb32 col;
					col.rgba = *p;
					vec4 col4; col4.a = { (float)col.b.b, (float)col.b.g, (float)col.b.r, (float)col.b.a };
					col4.v *= coeff;
					//ŖI CVTPS2DQ gǁACCasmgȂ@킩Ȃ
					*p++ = ((int)col4.a[2] << 16) | ((int)col4.a[1] << 8) | ((int)col4.a[0] << 0) | 0xff000000;
				}
			}
		} else {
			uint32_t* p = pixel_bits;
			memset(p, 0, w*h*sizeof(uint32_t));
			for (int y = 0; y < h; y++) {
				for (int x = 0; x < w; x++) {
					*p++ = 0xff000000;
				}
			}
		}

		int count = particle_count;
		int start = 0;
		int index = 0;
#ifdef GROUP4
		// 4_܂Ƃ߂ď
		v4sf mousex = { gravPoint_x, gravPoint_x, gravPoint_x, gravPoint_x };
		v4sf mousey = { gravPoint_y, gravPoint_y, gravPoint_y, gravPoint_y };
		v4sf force4 = { force, force, force, force };
		v4sf slow   = { 0.96, 0.96, 0.96, 0.96 };
		for (int i = 0; i < count-4; i+=4, start += 4) {
			vec4 posx; posx.a = { (*particles)[index+0], (*particles)[index+8+0], (*particles)[index+16+0], (*particles)[index+24+0] };
			vec4 posy; posy.a = { (*particles)[index+1], (*particles)[index+8+1], (*particles)[index+16+1], (*particles)[index+24+1] };
			vec4 vecx; vecx.a = { (*particles)[index+2], (*particles)[index+8+2], (*particles)[index+16+2], (*particles)[index+24+2] };
			vec4 vecy; vecy.a = { (*particles)[index+3], (*particles)[index+8+3], (*particles)[index+16+3], (*particles)[index+24+3] };

			vec4 diffx; diffx.v = mousex - posx.v;
			vec4 diffy; diffy.v = mousey - posy.v;
			vec4 weight; weight.a = { (*particles)[index+7], (*particles)[index+8+7], (*particles)[index+16+7], (*particles)[index+24+7] };
			vec4 sqx; sqx.v = diffx.v * diffx.v;
			vec4 sqy; sqy.v = diffy.v * diffy.v;
			vec4 acc; acc.v = force4 / (sqx.v + sqy.v);
			vec4 accx; accx.v = diffx.v * acc.v;
			vec4 accy; accy.v = diffy.v * acc.v;
			vec4 accxw; accxw.v = accx.v * weight.v;
			vec4 accyw; accyw.v = accy.v * weight.v;
			
			vecx.v += accx.v - accyw.v;
			vecy.v += accy.v + accxw.v;
			
			posx.v += vecx.v;
			posy.v += vecy.v;
			
			vecx.v *= slow;
			vecy.v *= slow;

			for (int j = 0; j < 4; j++) {
				float px = posx.a[j];
				float py = posy.a[j];
				
				if (px >= w) px -= w;
				if (px < 0)   px += w;
				if (py >= h) py -= h;
				if (py < 0)   py += h;
				
				int r = (int)(*particles)[index+j*8+4];// * px/w; r = r > 255 ? 255 : r;
				int g = (int)(*particles)[index+j*8+5];// * (1-px/w); g = g > 255 ? 255 : g;
				int b = (int)(*particles)[index+j*8+6];// * py/h; b = b > 255 ? 255 : b;
				uint32_t color = (r << 16) | (g << 8) | (b << 0);
				
				int ix = (int)px; ix = ix >= w ? w-1 : ix < 0 ? 0 : ix;
				int iy = (int)py; iy = iy >= h ? h-1 : iy < 0 ? 0 : iy;
				pixel_bits[w*iy + ix] = color | 0xff000000;

				(*particles)[index+j*8+0] = px;
				(*particles)[index+j*8+1] = py;
				(*particles)[index+j*8+2] = vecx.a[j];
				(*particles)[index+j*8+3] = vecy.a[j];
			}
			index += 8*4;
		}
#endif
		for (int i = start; i < count; i++) { // [
			double px = (*particles)[index+0];
			double py = (*particles)[index+1];
			double vx = (*particles)[index+2];
			double vy = (*particles)[index+3];
			
			double diff_x = gravPoint_x-px;
			double diff_y = gravPoint_y-py;
			double acc = (diff_x*diff_x + diff_y*diff_y);
			if (acc != 0.0) acc = force/acc; else acc = 0.0;
			double acc_x = diff_x*acc;
			double acc_y = diff_y*acc;
			vx += acc_x - acc_y*(*particles)[index+7];
			vy += acc_y + acc_x*(*particles)[index+7];
			
			px += vx;
			py += vy;
			
			vx *= 0.96;
			vy *= 0.96;
			
			if (px >= w)	px -= w;
			if (px < 0)		px += w;
			if (py >= h)	py -= h;
			if (py < 0)		py += h;

			int r = (int)(*particles)[index+4];
			int g = (int)(*particles)[index+5];
			int b = (int)(*particles)[index+6];
			
			int ix = (int)px; ix = ix >= w ? w-1 : ix < 0 ? 0 : ix;
			int iy = (int)py; iy = iy >= h ? h-1 : iy < 0 ? 0 : iy;
			pixel_bits[w*iy + ix] = (r << 16) | (g << 8) | b | 0xff000000;

			(*particles)[index+0] = px;
			(*particles)[index+1] = py;
			(*particles)[index+2] = vx;
			(*particles)[index+3] = vy;
			index += 8;
		}
		
		return pp::Var(1.0);
	}
	
}
